/**
 * Alfresco.CommentsControls component.
 *
 * Displays a list of comments.
 *
 * @namespace Alfresco
 * @class Alfresco.CommentsControls
 */
(function()
{
   /**
   * YUI Library aliases
   */
   var Dom = YAHOO.util.Dom;

   /**
    * CommentList constructor.
    *
    * @param {String} htmlId The HTML id of the parent element
    * @return {Alfresco.CommentsControls} The new Comment instance
    * @constructor
    */
   Alfresco.CommentsControls = function(htmlId)
   {
      Alfresco.CommentsControls.superclass.constructor.call(this, "Alfresco.CommentsControls", htmlId, ["editor", "paginator"]);

      /* Initialise prototype properties */
      this.editData =
      {
         viewDiv: null,
         row: -1,
         data: null,
         widgets: {}
      };

      YAHOO.Bubbling.on("permissionGroupsLoaded", this.onPermissionGroupsLoaded, this);

      return this;
   };

   YAHOO.extend(Alfresco.CommentsControls, Alfresco.component.Base,
   {
      /**
       * Object container for initialization options
       *
       * @property options
       * @type object
       */
      options:
      {
         /**
          * Node reference of the item to comment about
          */
         itemNodeRef: null,

         /**
          * Comment Topic to load.
          */
         commentTopic: "",

         /**
          * Number of items per page
          *
          * @property pageSize
          * @type int
          */
         pageSize: 5
      },


      /**
       * Comments data
       */
      commentsData: null,

      /**
       * Node permissions
       */
      nodePermissions: null,

      /**
       * Tells whether an action is currently ongoing.
       *
       * @property busy
       * @type boolean
       * @see _setBusy/_releaseBusy
       */
      busy: null,

      /**
       * Fired by YUI when parent element is available for scripting.
       * Component initialisation, including instantiation of YUI widgets and event listener binding.
       *
       * @method onReady
       */
      onReady: function CommentList_onReady()
      {
         var me = this;
         // YUI Paginator definition
         var paginator = new YAHOO.widget.Paginator(
         {
            containers: [this.id + "-paginator"],
            rowsPerPage: this.options.pageSize,
            initialPage: 1,
            template: this.msg("pagination.template"),
            pageReportTemplate: this.msg("pagination.template.page-report"),
            previousPageLinkLabel : this.msg("pagination.previousPageLinkLabel"),
            nextPageLinkLabel     : this.msg("pagination.nextPageLinkLabel")
         });
         paginator.subscribe('changeRequest', this.onPaginatorChange, this, true);
         paginator.set('recordOffset', 0);
         paginator.set('totalRecords', 0);
         paginator.render();
         this.widgets.paginator = paginator;
         this._loadCommentsList(0);
         YAHOO.Bubbling.fire("CommentsControl");
         YAHOO.Bubbling.fire("CommentsControl1");

      },

      /**
       * Called by the paginator when a user has clicked on next or prev.
       * Dispatches a call to the server and reloads the comment list.
       *
       * @method onPaginatorChange
       * @param state {object} An object describing the required page changing
       */
      onPaginatorChange : function CommentList_onPaginatorChange(state)
      {
         this._loadCommentsList(state.recordOffset);
      },

      /**
       * Loads the comments for the provided nodeRef and refreshes the ui
       */
      _loadCommentsList: function CommentList__loadCommentsList(startIndex)
      {
        // construct the url to call
        var url = YAHOO.lang.substitute(Alfresco.constants.PROXY_URI + "api/lyse/{nodeRef}/comments?reverse=true&commentTopic={commentTopic}",
        {
        	nodeRef: this.options.itemNodeRef.replace(":/", ""),
        	commentTopic: this.options.commentTopic
        });

        var executeAjaxRequest = function(obj, pageSize, startIndex, startDate, endDate) {
          // execute ajax request
          Alfresco.util.Ajax.request(
              {
                url: url,
                dataObj:
                {
                  startIndex: startIndex,
                  pageSize: pageSize,
                  reverse: true,
                  startDate: startDate,
                  endDate: endDate
                },
                successCallback:
                {
                  fn: obj.loadCommentsSuccess,
                  scope: obj
                },
                failureMessage: obj.msg("message.loadComments.failure")
              });
        };

        executeAjaxRequest(this, this.options.pageSize, startIndex);
      },
      /**
       * Loads the comments for the provided nodeRef and refreshes the ui
       */
      _loadAllCommentsListAndPrint: function CommentList__loadAllCommentsListAndPrint(startIndex)
      {
        // construct the url to call
        var url = YAHOO.lang.substitute(Alfresco.constants.PROXY_URI + "api/lyse/{nodeRef}/comments?reverse=true&commentTopic={commentTopic}",
        {
          nodeRef: this.options.itemNodeRef.replace(":/", ""),
          commentTopic: this.options.commentTopic
        });

        var executeAjaxRequest = function(obj, pageSize, startIndex, startDate, endDate) {
          // execute ajax request
          Alfresco.util.Ajax.request(
              {
                url: url,
                dataObj:
                {
                  startIndex: startIndex,
                  pageSize: pageSize,
                  reverse: true,
                  startDate: startDate,
                  endDate: endDate
                },
                successCallback:
                {
                  fn: obj.loadAllCommentsPrintSuccess,
                  scope: obj
                },
                failureMessage: obj.msg("message.loadComments.failure")
              });
        };
        executeAjaxRequest(this, this.options.pageSize, startIndex);
      },
      /**
       * Adjust amount of comments per page.
       */
      _setCommentsPerPageSize : function CommentsList_setCommentsPerPageSize(size){
          this.options.pageSize = size;
      },
       /**
       * Load all comments print ajax request success handler.
       */
      loadAllCommentsPrintSuccess: function CommentsList_loadCommentsSuccess(response)
      {
        var comments = response.json.items;
        YAHOO.util.Event.onContentReady(this.id, function(me) {
             // Get the elements to update
             var bodyDiv = Dom.get(me.id),
                 commentDiv = Dom.get(me.id + "-comments"),
                 submitButton = Dom.get(me.id + "-form-submit-button");

             // temporarily hide the container node
             bodyDiv.setAttribute("style", "display:none");

             // Update the list elements
             var html = '', i, j;
             for (i = 0, j = comments.length; i < j; i++) {
                 html += me.renderComment(i, comments[i], 0);
             }
             commentDiv.innerHTML = html;
             bodyDiv.removeAttribute("style");

             // keep a reference to the loaded data
             me.commentsData = comments;
             me.nodePermissions = response.json.nodePermissions;
             if (!me.nodePermissions.comment) {
                 submitButton.disabled = true;
             }
             me._updatePaginator(response.json.startIndex, response.json.total);
             window.print();
             me._setCommentsPerPageSize(5); // Reset page to 5 comments per page.
             me._loadCommentsList(0); // Refresh.
         }, this);
      },

      /**
       * Load comments ajax request success handler.
       */
      loadCommentsSuccess: function CommentsList_loadCommentsSuccess(response)
      {
         var comments = response.json.items;

         YAHOO.util.Event.onContentReady(this.id, function(me) {
             // Get the elements to update
             var bodyDiv = Dom.get(me.id),
                 commentDiv = Dom.get(me.id + "-comments"),
                 submitButton = Dom.get(me.id + "-form-submit-button");

             // temporarily hide the container node
             bodyDiv.setAttribute("style", "display:none");

             // Update the list elements
             var html = '', i, j;
             for (i = 0, j = comments.length; i < j; i++) {
                 html += me.renderComment(i, comments[i], 0);
             }
             commentDiv.innerHTML = html;
             bodyDiv.removeAttribute("style");

             // keep a reference to the loaded data
             me.commentsData = comments;
             me.nodePermissions = response.json.nodePermissions;
             if (!me.nodePermissions.comment) {
                 submitButton.disabled = true;
             }
             me._updatePaginator(response.json.startIndex, response.json.total);
             //#47433 - Custom fix, I suspect this is a bug in Alfresco. Support ticket raised here: https://myalfresco.force.com/support/apex/CustomViewCase?id=500D000001iL7ySIAS
             if(tinyMCE && tinyMCE.activeEditor && tinyMCE.isIE) {
               tinyMCE.activeEditor.isNotDirty = 1;
             }
         }, this);
      },
      /**
       * Called by loadCommentsSuccess when it has rendered the comments.
       * Since this componenent listens for the event "setCommentedNode" that can be displayed
       * before this component has created its own widgets and paginator it must wait until the paginator
       * has been created and then update it.
       *
       * @method updatePaginator
       * @param page {int} The page of comments in the paging list that is displayed
       * @param total {int} The totla number of comments in the paging
       */
      _updatePaginator: function CommentList__updatePaginator(page, total)
      {
         if (this.widgets && this.widgets.paginator)
         {
            this.widgets.paginator.set('recordOffset', page);
            this.widgets.paginator.set('totalRecords', total);
         }
         else
         {
            YAHOO.lang.later(100, this, this._updatePaginator, [page, total]);
         }
      },

      /**
       * Renders a comment element.
       * Each comment element consists of an edit and a view div.
       */
      renderComment: function CommentList_renderComment(index, data, level)
      {
         // add a div for the comment edit form
         var html = '';
         html += '<div id="' + this.id + '-comment-edit-' + index + '" class="hidden"></div>';

         // output the view
         var rowClass = index % 2 === 0 ? "even" : "odd";
         html += '<div class="comment ' + rowClass + '" id="' + this.id + '-comment-view-' + index + '">';
         html += this.renderCommentView(index, data, level);
         html += '</div>';

         return html;
      },

      /**
       * Permission group
       *
       * This never went into production, keeping fetching method if requirement pops up again.
       *
       */
      getPermissionGroups: function CommentList_getPermissionGroups(data)
      {

    	var nodeRef = new Alfresco.util.NodeRef(data.nodeRef);
	    Alfresco.util.Ajax.jsonGet(
	    {
 		     url: Alfresco.constants.PROXY_URI + "api/lyse/getPermissions/" + nodeRef.uri + "?excludeSystemGroups=true",
              successCallback:
              {
                fn: function (response){
    	              if (response.json !== undefined)
    	              {
    	            	var result = {nodeRef: data.nodeRef, permissionGroups: response.json.data};
    	                YAHOO.Bubbling.fire("permissionGroupsLoaded", result);
    	              }
    	         },
                 scope: this
              }
	 	});
      },

      /**
       * Permission groups loaded
       *
       * This never went into production, keeping fetching method if requirement pops up again.
       *
       */
      onPermissionGroupsLoaded: function CommentList_onPermissionGroupsLoaded(layer, args)
      {
         var permissionGroups = args[1].permissionGroups;
         var padlockImage=Dom.get("padlock-" + args[1].nodeRef);
         var titleString = "Groups with permission to read:\n";
         // Loop through all permission groups and output to padlockImage.title.
         for (var i = 0; i < permissionGroups.length; i++){
        	 titleString = titleString + permissionGroups[i].displayName + "\n";
         }
         padlockImage.title = titleString;

      },

      /**
       * Renders the content of the comment view div.
       */
      renderCommentView: function CommentList_renderCommentView(index, data, level)
      {
         var html = '';

         var padlockUrl = Alfresco.constants.URL_RESCONTEXT + 'components/images/padlock.gif';
         var padlockTitle = this.msg("comment.padlock.info");

         var commentVisibility = data.commentVisibility;

         // Different padlock color depending on visibility
         if (commentVisibility == "CIVIL"){
        	 padlockUrl = Alfresco.constants.URL_RESCONTEXT + 'components/images/padlockblue.gif';
        	 padlockTitle = this.msg("listconstraint.lysewf_shareRestrictionConstraint.shareRestriction.civil")
         }

         if (commentVisibility == "ENGINEERING"){
        	 padlockUrl = Alfresco.constants.URL_RESCONTEXT + 'components/images/padlockblue.gif';
        	 padlockTitle = this.msg("listconstraint.lysewf_shareRestrictionConstraint.shareRestriction.engineering")
         }

         if (commentVisibility == "INTERNAL"){
        	 padlockUrl = Alfresco.constants.URL_RESCONTEXT + 'components/images/padlock.gif';
        	 padlockTitle = this.msg("listconstraint.lysewf_shareRestrictionConstraint.shareRestriction.internal")
         }

         // comment info and content
         if (level === 0) {
            html += '<div class="nodeContent">';
         }
         else {
            html += YAHOO.lang.substitute('<div class="nodeContent" style="padding-left: {margin}px; background-image: url({imageUrl}); background-repeat: no-repeat;">', {
               margin: level * 20,
               imageUrl: Alfresco.constants.URL_RESCONTEXT + "components/images/reply-to-discussion-16.png"
            });
         }

         if (data.permissions.inheritsFromParent === false){
        	 html += '<div class="content yuieditor"><img id="padlock-'+ data.nodeRef +'" src="'+padlockUrl+'" title="'+padlockTitle+'"/></div>'
         }

         html += '<div class="userLink">' + Alfresco.util.people.generateUserLink(data.author);
         html += ' ('+Alfresco.util.formatDate(data.createdOn)+'), ';
         html += '</div>';
         html += '<div class="content yuieditor lysecomment">' + data.content + '</div>';
         html += '</div>';

         if (data.replies && data.replies.length > 0) {
            for (i = 0; i < data.replies.length; i++) {
               html += this.renderComment(index, data.replies[i], level+1);
            }
         }
         return html;
      }
   });
})();
