/**
 * Query Lists: QueryLists component.
 * 
 * Displays a list of querylists
 * 
 * @namespace Alfresco
 * @class Alfresco.component.QueryLists
 */
(function()
{
   /**
    * YUI Library aliases
    */
   var Dom = YAHOO.util.Dom,
      Event = YAHOO.util.Event,
      Selector = YAHOO.util.Selector,
      Bubbling = YAHOO.Bubbling;

   /**
    * Alfresco Slingshot aliases
    */
   var $html = Alfresco.util.encodeHTML,
      $combine = Alfresco.util.combinePaths;

   /**
    * QueryLists constructor.
    * 
    * @param htmlId {String} The HTML id of the parent element
    * @return {Alfresco.component.QueryLists} The new QueryLists instance
    * @constructor
    */
   Alfresco.component.QueryLists = function(htmlId)
   {
      Alfresco.component.QueryLists.superclass.constructor.call(this, "Alfresco.component.QueryLists", htmlId, ["button", "container"]);


      // Initialise prototype properties
      this.queryLists = {};
      this.queryListsLength = null;
      this.containerNodeRef = null;
      this.listTypeTitles = null;
      
      return this;
   };
   
   /**
    * Extend from Alfresco.component.Base
    */
   YAHOO.extend(Alfresco.component.QueryLists, Alfresco.component.Base,
   {
      /**
       * Object container for initialization options
       *
       * @property options
       * @type object
       */
      options:
      {
         /**
           * Current siteId.
           * 
           * @property siteId
           * @type string
           * @default ""
           */
          siteId: "",

          /**
           * ContainerId representing root container.
           *
           * @property containerId
           * @type string
           * @default "queryLists"
           */
          containerId: "queryLists",
          
          /**
           * ListId representing currently selected list
           *
           * @property listId
           * @type string
           */
          listId: "",

          /**
           * List types when creating new Data Lists
           *
           * @property listTypes
           * @type Array
           */
          listTypes: []
      },

      /**
       * Data Lists metadata retrieved from the Repository
       *
       * @param queryLists
       * @type Object
       */
      queryLists: null,

      /**
       * Number of data lists in the Repository
       *
       * @param queryListsLength
       * @type Object
       */
      queryListsLength: null,

      /**
       * NodeRef of the Data Lists container retrieved from the Repository
       *
       * @param containerNodeRef
       * @type Object
       */
      containerNodeRef: null,

      /**
       * List title look-up from listType
       *
       * @property listTypeTitles
       * @type Object
       */
      listTypeTitles: null,

      /**
       * Fired by YUI when parent element is available for scripting.
       *
       * @method onReady
       */
      onReady: function QueryLists_onReady()
      {
         this.widgets.newList = Alfresco.util.createYUIButton(this, "newListButton", this.onNewList,
         {
            disabled: true
         });
         
         // Retrieve the lists from the specified Site
         this.populateQueryLists(
         {
            fn: function QueryLists_onReady_callback()
            {
               this.renderQueryLists();

               // Select current list, if relevant
               if (this.options.listId.length > 0)
               {
                  Bubbling.fire("activeDataListChanged",
                  {
                     dataList: this.queryLists[this.options.listId],
                     scrollTo: true
                  });
               }
               else
               {
                  YAHOO.Bubbling.fire("hideFilter");
               }
               
               if (this.dataListsLength === 0 || window.location.hash === "#new")
               {
                  this.widgets.newList.fireEvent("click");
               }
            },
            scope: this
         });
         
      },
      
    
      /**
       * Retrieves the Query Lists from the Repo
       *
       * @method populateQueryLists
       * @param callback {Object} Optional callback literal {fn, scope, obj} whose function is invoked once list has been retrieved
       */
      populateQueryLists: function QueryLists_populateQueryLists(p_callback)
      {
         /**
          * Success handler for Query Lists request
          * @method fnSuccess
          * @param response {Object} Ajax response object literal
          * @param obj {Object} Callback object from original function call
          */
         var fnSuccess = function QueryLists_pDL_fnSuccess(response, p_obj)
         {
            var lists = response.json.querylists,
               list;
            
            this.queryLists = {};
            //this.containerNodeRef = new Alfresco.util.NodeRef(response.json.container);
            //this.widgets.newList.set("disabled", true);
            
            for (var i = 0, ii = lists.length; i < ii; i++)
            {
               list = lists[i];
               this.queryLists[list.name] = list;
            }
            this.queryListsLength = lists.length;
            
            if (p_callback && (typeof p_callback.fn == "function"))
            {
               p_callback.fn.call(p_callback.scope || this, p_callback.obj);
            }
         };

         /**
          * Failure handler for Query Lists request
          * @method fnFailure
          * @param response {Object} Ajax response object literal
          */
         var fnFailure = function QueryLists_pDL_fnFailure(response)
         {
            if (response.status == 401)
            {
               // Our session has likely timed-out, so refresh to offer the login page
               window.location.reload();
            }
            else
            {
               this.queryLists = null;
               //this.containerNodeRef = null;
               //this.widgets.newList.set("disabled", true);
               var errorMsg = "";
               try
               {
                  errorMsg = $html(YAHOO.lang.JSON.parse(response.responseText).message);
               }
               catch(e)
               {
                  errorMsg = this.msg("message.error-unknown");
               }
            }
         };
         
         Alfresco.util.Ajax.jsonGet(
         {
            url: Alfresco.constants.PROXY_URI + "api/lyse/querylists/site/" + encodeURIComponent(this.options.siteId),
            successCallback:
            {
               fn: fnSuccess,
               obj: p_callback,
               scope: this
            },
            failureCallback:
            {
               fn: fnFailure,
               scope: this
            }
         });
      },
      
     
      
      renderQueryLists: function QueryLists_renderQueryLists(p_callback)
      {

		 var me = this,
         listsContainer = Dom.get(this.id + "-lists"),
         selectedClass = "selected";
         
         listsContainer.innerHTML = "";
         
         /**
          * Click handler for selecting Data List
          * @method fnOnClick
          */
         var fnOnClick = function QueryLists_renderQueryLists_fnOnClick()
         {
            return function QueryLists_renderQueryLists_onClick()
            {
               var lis = Selector.query("li", listsContainer);
               Dom.removeClass(lis, selectedClass);
               Dom.addClass(this, selectedClass);
               
               return true;
            };
         };

         try
         {
            var lists = this.queryLists,
               list,
               permissions,
               elHighlight = null,
               container, el, elEdit, elDelete, elLink, elText;

            if (this.queryListsLength === 0)
            {
               listsContainer.innerHTML = '<div class="no-lists">' + this.msg("message.no-lists") + '</div>';
            }
            else
            {
               container = document.createElement("ul");
               listsContainer.appendChild(container);

               // Create the DOM structure: <li onclick><a class='filter-link' title href><span class='edit' onclick></span><span class='delete' onclick></span>"text"</a></li>
               for (var index in lists)
               {
                  if (lists.hasOwnProperty(index))
                  {
                     list = lists[index];
                     permissions = list.permissions;
                     
                     // Build the DOM elements
                     el = document.createElement("li");
                     el.onclick = fnOnClick();

                     elLink = document.createElement("a");
                     elLink.className = "filter-link";
                     elLink.title = list.description;
                     elLink.href = "query-lists?list=" + $html(list.name);
                     elText = document.createTextNode(list.title);

                     // Build the DOM structure with the new elements
                     elLink.appendChild(elText);
                     el.appendChild(elLink);
                     container.appendChild(el);

                     // Mark current list as selected
                     if (list.name == this.options.listId)
                     {
                        Dom.addClass(el, "selected");
                     }
                     
                     // Make a note of a highlight request match
                     if (list.name == this.options.listId)
                     {
                        elHighlight = el;
                     }
                  }
               }
               
               if (elHighlight)
               {
                  Alfresco.util.Anim.pulse(elHighlight);
               }
            }
         }
         catch(e)
         {
            listsContainer.innerHTML = '<span class="error">' + this.msg("message.error-unknown") + '</span>';
            console.log(e);
         }
         
         if (p_callback && (typeof p_callback.fn == "function"))
         {
            p_callback.fn.call(p_callback.scope || this, p_callback.obj);
         }

  }
 });
})();