//@overridden projects/slingshot/source/web/components/workflow/workflow-form.js
/**
 * Copyright (C) 2005-2010 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * WorkflowForm component.
 *
 * The workflow details page form is actually a form display of the workflow's start task and data form the workflow itself.
 * To be able to display all this information the following approach is taken:
 *
 * 1. The page loads with a url containing the workflowId as an argument.
 * 2. Since we actually want to display the start task the data-loader component has been bound in to the bottom of the page,
 *    instructed to load detailed workflow data based on the workflowId url argument,
 *    so we can get the startTaskInstanceId needed to request the form.
 * 3. A dynamically/ajax loaded form is brought in using the startTaskInstanceId which gives us a start task form with the
 *    "More Info", "Roles" and "Items" sections.
 * 4. However we shall also display info from the workflow itsel, so once the form is loaded and inserted in to the Dom,
 *    the additional sections "Summary", "General", "Current Tasks" & "Workflow History" are inserted inside the form.
 *
 * @namespace Lyse.component
 * @class Lyse.component.WorkflowForm
 */

if (!Lyse) {
   var Lyse = {};
}
if (!Lyse.component) {
   Lyse.component = {};
}

(function()
{
   /**
    * YUI Library aliases
    */
   var Dom = YAHOO.util.Dom,
      Event = YAHOO.util.Event,
      Selector = YAHOO.util.Selector;

   /**
    * Alfresco Slingshot aliases
    */
   var $html = Alfresco.util.encodeHTML,
      $siteURL = Alfresco.util.siteURL,
      $userProfileLink = Alfresco.util.userProfileLink;

   /**
    * Some times the dom element is not ready when the data is. This happens
    * when target element is part of a XHR request response.
    *
    * This hack checks if the element has appeared in the DOM,
    * or tries again one second later.
    */
   function fillElement(id,data){
       var formEl = Dom.get(id);
       if (formEl) formEl.innerHTML = data;
       else {
    	   setTimeout(function(){ fillElement(id,data); }, 1000);
       }
   }

   /**
    * WorkflowForm constructor.
    *
    * @param {String} htmlId The HTML id of the parent element
    * @return {Lyse.component.WorkflowForm} The new WorkflowForm instance
    * @constructor
    */
   Lyse.component.WorkflowForm = function WorkflowForm_constructor(htmlId)
   {

      Lyse.component.WorkflowForm.superclass.constructor.superclass.constructor.call(this, "Lyse.component.WorkflowForm", htmlId, ["button", "container", "datasource", "datatable"]);
      this.isReady = false;
      this.workflow = null;
      this.currentTasks = [];
      this.historyTasks = [];
      this.datalistNodeRef = null;
      this.nodeRef = Alfresco.util.getQueryStringParameter("nodeRef", document.location.href);
      this.workflowId = Alfresco.util.getQueryStringParameter("workflowId", document.location.href);

      if (this.workflowId != null) {
      /* Decoupled event listeners */
         YAHOO.Bubbling.on("workflowDetailedData", this.onWorkflowDetailedData, this);
      }
      else if (this.nodeRef != null) {
         this.showDatalistForm(this.nodeRef, function(response) {
        	fillElement(this.id + "-body",response.serverResponse.responseText);
         });
      }
   };

   YAHOO.extend(Lyse.component.WorkflowForm, Alfresco.component.WorkflowForm,
   {


      /**
       * @method _displayWorkflowForm
       * @private
       */
      _loadWorkflowForm: function WF__loadWorkflowForm()
      {
         if (this.isReady && this.workflow) {
               // Split the task list in current and history tasks and save the most recent one
               var tasks = this.workflow.tasks, recentTask;
               for (var i = 0, il = tasks.length; i < il; i++) {
                  // Find the related data list nodeRef
                  if (tasks[i].id === this.workflow.startTaskInstanceId) {
                    this.datalistNodeRef = tasks[i].properties.lysewf_relatedDatalistItem;
                    // #48342 In tenants they like to add @tenant-name@ in the nodeRef, this causes issues.
                    this.datalistNodeRef = this.datalistNodeRef.replace(/@.*@/g,"");
                  }
                  if (tasks[i].state == "COMPLETED") {
                     this.historyTasks.push(tasks[i]);
                  }
                  else {
                     this.currentTasks.push(tasks[i]);
                  }
               }

               var sortByDate = function (dateStr1, dateStr2) {
                  var date1 = Alfresco.util.fromISO8601(dateStr1),
                      date2 = Alfresco.util.fromISO8601(dateStr2);
                  if (date1 && date2) {
                     return date1 < date2 ? 1 : -1;
                  }
                  else {
                     return !date1 ? 1 : -1;
                  }
               };

               // Sort tasks by completion date
               this.currentTasks.sort(function (task1, task2) {
                  return sortByDate(task1.properties.bpm_dueDate, task2.properties.bpm_dueDate);
               });

               // Sort tasks by completion date
               this.historyTasks.sort(function (task1, task2) {
                  return sortByDate(task1.properties.bpm_completionDate, task2.properties.bpm_completionDate);
               });

               // Load data list view form (if present)
               var dlNodeRef = Alfresco.util.getQueryStringParameter("nodeRef", document.location.href) || this.datalistNodeRef;
               if (dlNodeRef && !Alfresco.util.getQueryStringParameter("hideDL", document.location.href)) {
                  this.showDatalistForm(dlNodeRef, this.onWorkflowFormLoaded);
               } else {
                  this.onWorkflowFormLoaded({serverResponse: {responseText: "<div class='form-container'><div class='form-fields'></div></div>"}});
               }
         }
      },

      /**
       * Called when a workflow form has been loaded.
       * Will insert the form in the Dom.
       *
       * @method onWorkflowFormLoaded
       * @param response {Object}
       */
      onWorkflowFormLoaded: function WorkflowForm_onWorkflowFormLoaded(response)
      {
         // Unsubscribe from the event to prevent invocation of this method when another workflow form is loaded for another dialog
         YAHOO.Bubbling.unsubscribe("workflowDetailedData");

         // Insert the form html
     	 fillElement(this.id + "-body",response.serverResponse.responseText);

         // Insert the summary & general sections in the top of the form
         var formFieldsEl = Selector.query(".form-fields", this.id, true);

         // Create header and data table elements
         var currentTasksContainerEl = Dom.get(this.id + "-currentTasks-form-section"),
            currentTasksTasksEl = Selector.query("div", currentTasksContainerEl, true);

         // DataTable column definitions for current tasks
         var currentTasksColumnDefinitions =
         [
            { key: "name", label: this.msg("column.type"), formatter: this.bind(this.renderCellType) },
            { key: "owner", label: this.msg("column.assignedTo"), formatter: this.bind(this.renderCellOwner) },
            { key: "pooledActors", label: this.msg("column.pooledActors"), formatter: this.bind(this.renderCellPooledActors) },
            { key: "id", label: this.msg("column.dueDate"), formatter: this.bind(this.renderCellDueDate) }
         ];

         // Create current tasks data table filled with current tasks
         var currentTasksDS = new YAHOO.util.DataSource(this.currentTasks,
         {
            responseType: YAHOO.util.DataSource.TYPE_JSARRAY
         });
         this.widgets.currentTasksDataTable = new YAHOO.widget.DataTable(currentTasksTasksEl, currentTasksColumnDefinitions, currentTasksDS,
         {
            MSG_EMPTY: this.msg("label.noTasks")
         });

         // DataTable column definitions workflow history
         var historyColumnDefinitions =
         [
            { key: "name", label: this.msg("column.type"), formatter: this.bind(this.renderCellType) },
            { key: "owner", label: this.msg("column.completedBy"), formatter: this.bind(this.renderCellCompletedBy) },
            { key: "id", label: this.msg("column.dateCompleted"), formatter: this.bind(this.renderCellDateCompleted) },
            { key: "state", label: this.msg("column.outcome"), formatter: this.bind(this.renderCellOutcome) }/*,
            { key: "properties", label: this.msg("column.comment"), formatter: this.bind(this.renderCellComment) }*/
         ];

         // Create header and data table elements
         var historyContainerEl = Dom.get(this.id + "-workflowHistory-form-section"),
            historyTasksEl = Selector.query("div", historyContainerEl, true);

         // Create workflow history data table filled with history tasks
         var workflowHistoryDS = new YAHOO.util.DataSource(this.historyTasks,
         {
            responseType: YAHOO.util.DataSource.TYPE_JSARRAY
         });
         this.widgets.historyTasksDataTable = new YAHOO.widget.DataTable(historyTasksEl, historyColumnDefinitions, workflowHistoryDS,
         {
            MSG_EMPTY: this.msg("label.noTasks")
         });

         // Add task buttons
         if (this.currentTasks && this.currentTasks.length > 0) {
            //var buttons = Selector.query(".buttons");
            var buttons = document.getElementById("task-buttons");
            if (buttons) {
               for (i = 0 ; i < this.currentTasks.length; i++) {
                  var task = this.currentTasks[i];
                  // These are already implemented in getShareTasks: function (nodeRef). Reason, is this function will reference same task id which causes problems.
                  var checkTasks = task.name == "lysewf:cdlCompanyReviewUserTask" || task.name == "lysewf:cdlCompanyDiscussionUserTask";

                  if (task.isEditable && !checkTasks) {
                     var taskButton = document.createElement("button");
                     taskButton.innerHTML = "Task: " + task.title;
                     taskButton.setAttribute("class", "yui-button yui-push-button");
                     var listItem = document.createElement("li");
                     listItem.appendChild(taskButton);
                     buttons.appendChild(listItem);
                     var button = new YAHOO.widget.Button(taskButton);

                     if(window.location.href) {
	                     if(window.location.href.indexOf("lyse-datalist-details") !== -1){
                			button.removeClass("yui-button");
	                     }
                     }

                     console.log(task);
                     console.log("task id (som nog är fel?): " + task.id)
                     var url = YAHOO.lang.substitute("{taskUrl}&workflowId={workflowId}", {
                         taskUrl: this._getTaskUrl("lyse-task-edit", task.id),
                         workflowId: encodeURIComponent(Alfresco.util.getQueryStringParameter("workflowId", document.location.href))
                      });
                     console.log(url)
                     button.on("click",(function()
                     {
                        document.location = url;
                     }).bind(this));
                  }
               }
            }
         }

        // #44967 Add Civil MQ special task button
        Alfresco.util.Ajax.jsonGet({
          url: Alfresco.constants.PROXY_URI + "api/metadata?nodeRef=" + this.datalistNodeRef,
          responseContentType: "application/json",
          successCallback: {
            fn: function (response) {
              if (response.json.properties["{http://ns.lyse.no/model/1.0}civilMqStatus"] === "approved") {
                var buttons = document.getElementById("task-buttons");
                if (buttons) {
                  var taskButton = document.createElement("button");
                  taskButton.innerHTML = "Upload new bill";
                  taskButton.setAttribute("class", "yui-button yui-push-button");
                  var listItem = document.createElement("li");
                  listItem.appendChild(taskButton);
                  buttons.appendChild(listItem);
                  var button = new YAHOO.widget.Button(taskButton);
                  button.on("click",(function() {
                    Alfresco.util.Ajax.jsonGet( {
                      url: Alfresco.constants.PROXY_URI + "lyse/api/civil-mq-start-new-version-task?nodeRef=" + this.datalistNodeRef,
                      successCallback: {
                        fn: function () {
                          Alfresco.util.PopupManager.displayMessage( {
                              text: this.msg("actions.start-new-mq-version.success")
                          });
                          var nodeRefs =  [this.datalistNodeRef];
                          Alfresco.util.Ajax.jsonPost({
                            url : Alfresco.constants.PROXY_URI + "/api/lyse/datatable-metadata",
                            dataObj: nodeRefs,
                            responseContentType : "application/json",
                            successCallback : {
                              fn : function(response) {
                                //Reload
                                location.reload();
                                var taskId = response.json[0].workflowTasks[0].id;
                                var workflowId = response.json[0].workflowTasks[0].workflowId;
                                window.open(Alfresco.constants.URL_PAGECONTEXT +'lyse-task-edit?taskId=' + taskId + '&workflowId=' + workflowId, "_blank");
                              },
                              scope : this,
                            }
                          })
                        },
                        scope: this
                      },
                      failureCallback: {
                        fn: function (response) {
                          Alfresco.util.PopupManager.displayPrompt(
                            {
                              title: this.msg("actions.start-new-mq-version.failure"),
                              text: response.json.message
                            });
                        },
                        scope: this
                      }
                    });
                  }).bind(this));
                }
              }
            },
            scope: this
          },
        });

        // Put the workflow history below the data list view form
        Selector.query(".form-fields", this.id, true).appendChild(historyContainerEl);

        if (this.workflow.diagramUrl)
         {
            var diagramContainerEl = Dom.get(this.id + "-workflowDiagram-section"),
                diagramEl = Selector.query("div", diagramContainerEl, true);

            diagramEl.innerHTML = '<img src="' + Alfresco.constants.PROXY_URI + this.workflow.diagramUrl + '" style="max-width: inherit;"></img>';
            Selector.query(".form-fields", this.id, true).appendChild(diagramContainerEl);
            //formEl.appendChild(diagramContainerEl);
         }

         // Fire event so other components knows the form finally has been loaded
         YAHOO.Bubbling.fire("workflowFormReady", this);
      },
      /**
       *
       * @param nodeRef
       * @param fnSuccessCallback
       * called when either nodeRef or WorkflowId represented as url query param.
       */
      /**
      *
      * @param nodeRef
      * @param fnSuccessCallback
      * called when either nodeRef or WorkflowId represented as url query param.
      */
     showDatalistForm: function showDatalistForm(nodeRef, fnSuccessCallback)
     {
     var me = this;
   	 var formsCall = function(shortname) {
	         Alfresco.util.Ajax.request(
		             {
		                url: Alfresco.constants.URL_SERVICECONTEXT + "components/form",
		                dataObj: {
		                   htmlid: this.id + "-DatalistForm-" + Alfresco.util.generateDomId(),
		                   itemKind: "node",
		                   itemId: nodeRef,
		                   mode: "view",
		                   formId: "datalist-details",
		                   formUI: false,
		                   site: shortname
		                },
		                successCallback: {
		                   fn: fnSuccessCallback,
		                   scope: me
		                },
		                failureMessage: Alfresco.util.message("message.failure"),
		                scope: me,
		                execScripts: true
		             });

   	 };

   	 if (Alfresco.constants.SITE != null && Alfresco.constants.SITE.length > 0)
     	 {
   		 formsCall(Alfresco.constants.SITE);
	     }else {
	      	 // If the site constant is null we need to find out in which site the node
	      	 // is stored, since cdlDataList-items have different forms config depending on
	      	 // originating site (engineering doc or contractors doc)
	      	 Alfresco.util.Ajax.jsonGet({
			    	  url: Alfresco.constants.PROXY_URI + "/api/lyse/" + Alfresco.util.NodeRef(nodeRef).uri + "/site-shortname",
			    	  responseContentType : "application/json",
			    	  successCallback:
			    	  {
			    		  fn: function (response){
					         formsCall(response.json.shortName);
			    		  },
			    		  scope: me
			    	  }
			 });
	     }
     },


   /**
       * Called when view workflow diagram button is clicked.
       * WIll display the workflow's diagram.
       */
      viewWorkflowDiagram: function()
      {
         if (this.workflow.diagramUrl)
         {
            Alfresco.Lightbox.show({ src: Alfresco.constants.PROXY_URI + this.workflow.diagramUrl });
         }
      },

      /**
       * Render task type as link
       *
       * @method renderCellType
       * @param elCell {object}
       * @param oRecord {object}
       * @param oColumn {object}
       * @param oData {object|string}
       */
      renderCellType: function WorkflowForm_renderCellType(elCell, oRecord, oColumn, oData)
      {
         var task = oRecord.getData();
         if (task.isEditable)
         {
            elCell.innerHTML = '<a href="' + this._getTaskUrl("task-edit", oRecord.getData("id")) + '" title="' + this.msg("link.title.task-edit") + '">' + $html(oRecord.getData("title")) + '</a>';
         }
         else
         {
            elCell.innerHTML = '<a href="' + this._getTaskUrl("task-details", oRecord.getData("id")) + '" title="' + this.msg("link.title.task-details") + '">' + $html(oRecord.getData("title")) + '</a>';
         }
      },

      /**
       * Render task owner as link
       *
       * @method renderCellOwner
       * @param elCell {object}
       * @param oRecord {object}
       * @param oColumn {object}
       * @param oData {object|string}
       */
      renderCellOwner: function WorkflowForm_renderCellOwner(elCell, oRecord, oColumn, oData)
      {
         var owner = oRecord.getData("owner");
         if (owner != null && owner.userName)
         {
            var displayName = this.msg("field.owner", owner.firstName, owner.lastName);
            elCell.innerHTML = $userProfileLink(owner.userName, owner.firstName && owner.lastName ? displayName : null, null, !owner.firstName);
         }
         else {
            elCell.innerHTML = this.msg("label.none");
         }
      },

      /**
       * Render pooled actors.
       *
       * @method renderCellPooledActors
       * @param elCell {object}
       * @param oRecord {object}
       * @param oColumn {object}
       * @param oData {object|string}
       */
      renderCellPooledActors: function WorkflowForm_renderCellPooledActors(elCell, oRecord, oColumn, oData)
      {
         var isPooled = oRecord.getData("isPooled");
         var properties = oRecord.getData("properties");
         var pooledActors = properties !== undefined ? properties["bpm_pooledActors"] : undefined;
         if (isPooled != null && isPooled && properties !== undefined && pooledActors !== undefined && pooledActors.length) {
            for (i = 0; i < pooledActors.length; i++) {

               // Find the display name for the group
               Alfresco.util.Ajax.request(
                   {
                      url: YAHOO.lang.substitute("{context}api/metadata?nodeRef={nodeRef}",
                          {
                             context: Alfresco.constants.PROXY_URI,
                             nodeRef: pooledActors[i]
                          }),
                      dataObj: {},
                      successCallback: {
                         fn: function renderCellPooledActors_successCallback(response) {
                            elCell.innerHTML = (elCell.innerHTML.length > 0 ? elCell.innerHTML + ", " : "") + response.json.properties["{http://www.alfresco.org/model/content/1.0}authorityDisplayName"];
                         },
                         scope: this,
                         obj: null
                      },
                      failureMessage: Alfresco.util.message("message.failure"),
                      scope: this
                   });
            }
         }
         else {
            elCell.innerHTML = this.msg("label.none");
         }
      },

      /**
       * Render task completer (= owner) as link when the task actually has been Completed (eg. not the case when
       * parallel tasks aren't actually completed due to loop-stop was condition met).
       *
       * @method renderCellOwner
       * @param elCell {object}
       * @param oRecord {object}
       * @param oColumn {object}
       * @param oData {object|string}
       */
      renderCellCompletedBy: function WorkflowForm_renderCellCompletedBy(elCell, oRecord, oColumn, oData)
      {
    	  var status = oRecord.getData("properties").bpm_status;

    	  // Value based on list 'bpm:allowedStatus' in bpmModel.xml
    	  if(status != null && status != "Completed")
    	  {
             elCell.innerHTML = this.msg("label.none");
    	  }
    	  else
    	  {
             var creator = oRecord.getData("creator");
             if (creator == null)
             {
                creator = oRecord.getData("owner");
             }
             if (creator != null && creator.userName)
             {
                var displayName = this.msg("field.owner", creator.firstName, creator.lastName);
                elCell.innerHTML = $userProfileLink(creator.userName, creator.firstName && creator.lastName ? displayName : null, null, !creator.firstName);
             }
             else
             {
                elCell.innerHTML = this.msg("label.none");
             }
          }
      },

      /**
       * Render task completed date
       *
       * @method TL_renderCellSelected
       * @param elCell {object}
       * @param oRecord {object}
       * @param oColumn {object}
       * @param oData {object|string}
       */
      renderCellDateCompleted: function WorkflowForm_renderCellDateCompleted(elCell, oRecord, oColumn, oData)
      {
         var completionDate = Alfresco.util.fromISO8601(oRecord.getData("properties").bpm_completionDate);
         elCell.innerHTML = Alfresco.util.formatDate(completionDate);
      },

      /**
       * Render task due date
       *
       * @method renderCellDueDate
       * @param elCell {object}
       * @param oRecord {object}
       * @param oColumn {object}
       * @param oData {object|string}
       */
      renderCellDueDate: function WorkflowForm_renderCellDueDate(elCell, oRecord, oColumn, oData)
      {
         var dueISODate = oRecord.getData("properties").bpm_dueDate;
         if (dueISODate !== null)
         {
            var dueDate = Alfresco.util.fromISO8601(dueISODate);
            elCell.innerHTML = Alfresco.util.formatDate(dueDate, "defaultDateOnly");
         }
         else
         {
            elCell.innerHTML = this.msg("label.none");
         }
      },

      /**
       * Render task status
       *
       * @method TL_renderCellSelected
       * @param elCell {object}
       * @param oRecord {object}
       * @param oColumn {object}
       * @param oData {object|string}
       */
      renderCellStatus: function WorkflowForm_renderCellStatus(elCell, oRecord, oColumn, oData)
      {
         var status = oRecord.getData("properties").bpm_status;
         if (oRecord.getData("propertyLabels") && Alfresco.util.isValueSet(oRecord.getData("propertyLabels").bpm_status, false))
         {
            status = oRecord.getData("propertyLabels").bpm_status;
         }

         elCell.innerHTML = $html(status);
      },

      /**
       * Render task outcome
       *
       * @method renderCellOutcome
       * @param elCell {object}
       * @param oRecord {object}
       * @param oColumn {object}
       * @param oData {object|string}
       */
      renderCellOutcome: function WorkflowForm_renderCellOutcome(elCell, oRecord, oColumn, oData)
      {
     	 var outcomeLabel = oRecord.getData("outcome");

     	 if (Alfresco.util.isValueSet(oRecord.getData("properties").bpm_outcomePropertyName, false)){
     		outcomePropertyName = oRecord.getData("properties").bpm_outcomePropertyName.replace(":", "_");
            if (oRecord.getData("properties")[outcomePropertyName] && oRecord.getData("propertyLabels")[outcomePropertyName]) {
               outcomeLabel = oRecord.getData("propertyLabels")[outcomePropertyName];
            }
     	 }

         elCell.innerHTML = $html(outcomeLabel);
      },

      /**
       *  Render task comment
       *
       * @method renderCellComment
       * @param elCell {object}
       * @param oRecord {object}
       * @param oColumn {object}
       * @param oData {object|string}
       */
      renderCellComment: function WorkflowForm_renderCellComment(elCell, oRecord, oColumn, oData)
      {
         elCell.innerHTML = $html(oRecord.getData("properties").bpm_comment);
      },

      /**
       * Render actions available for current tasks
       *
       * @method renderCellCurrentTasksActions
       * @param elCell {object}
       * @param oRecord {object}
       * @param oColumn {object}
       * @param oData {object|string}
       */
      renderCellCurrentTasksActions: function WorkflowForm_renderCellCurrentTasksActions(elCell, oRecord, oColumn, oData)
      {
         var task = oRecord.getData();
         elCell.innerHTML += '<a href="' + this._getTaskUrl("task-details", task.id) + '" class="task-details" title="' + this.msg("link.title.task-details") + '">&nbsp;</a>';
         if (task.isEditable)
         {
            elCell.innerHTML += '<a href="' + this._getTaskUrl("task-edit", task.id) + '" class="task-edit" title="' + this.msg("link.title.task-edit") + '">&nbsp;</a>';
         }
      }

   });

   var showWorkflowHistory = function () {
     var historyContainerEl = Dom.get("workflowHistory-table");
     var diagramButton = Dom.get("toggleWorkflowHistory-button");

     var display = historyContainerEl.style.display;
     if(display == "none") {
       historyContainerEl.style.display = 'block';
       diagramButton.innerHTML = "Hide workflow history";
     } else {
       historyContainerEl.style.display = 'none';
       diagramButton.innerHTML = "Show workflow history";
     }

   };
   var showWorkflowDiagram = function () {
     var diagramContainerEl = Dom.get("workflowDiagram-table");
     var diagramButton = Dom.get("toggleWorkflowDiagram-button");

     var display = diagramContainerEl.style.display;
     if(display == "none") {
       diagramContainerEl.style.display = 'block';
       diagramButton.innerHTML = "Hide workflow diagram";
     } else {
       diagramContainerEl.style.display = 'none';
       diagramButton.innerHTML = "Show workflow diagram";
     }
   };

   YAHOO.util.Event.onDOMReady(function () {
     // We don't want show/hide buttons if the datalist is hidden
     if(!Alfresco.util.parseURL().queryParams.hideDL){
       // Default hide some elements
       var historyContainerEl = Dom.get("workflowHistory-table");
       var diagramContainerEl = Dom.get("workflowDiagram-table");
       historyContainerEl.style.display = 'none';
       diagramContainerEl.style.display = 'none';

       var oToggleWorkflowHistoryButton = new YAHOO.widget.Button({
         id: "toggleWorkflowHistory",
         type: "button",
         label: "Show workflow history",
         container: "hide-workflow-history-container"
       });

       var oToggleWorkflowDiagramButton = new YAHOO.widget.Button({
         id: "toggleWorkflowDiagram",
         type: "button",
         label: "Show workflow diagram",
         container: "hide-workflow-diagram-container"
       });

       oToggleWorkflowHistoryButton.on("click", showWorkflowHistory);
       oToggleWorkflowDiagramButton.on("click", showWorkflowDiagram);
     }
     else
     {
       //hide workflow-diagram when hiding datalist
       document.getElementsByClassName("workflow-diagram")[0].style.display = 'none';
     }
   });

})();
