/**
 * Copyright (C) 2005-2010 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * CreateDatalist module
 *
 * A dialog for creating sites
 *
 * @namespace Alfresco.module
 * @class Alfresco.module.CreateDatalist
 */
(function()
{

   var Dom = YAHOO.util.Dom,
      Event = YAHOO.util.Event,
      Element = YAHOO.util.Element,
      KeyListener = YAHOO.util.KeyListener;

   /**
    * CreateDatalist constructor.
    *
    * @param htmlId {string} A unique id for this component
    * @return {Alfresco.CreateDatalist} The new DocumentList instance
    * @constructor
    */
   Alfresco.module.CreateDatalist = function(instanceId, container)
   {

      var instance = Alfresco.util.ComponentManager.get(this.id);
      if (instance !== null)
      {
         throw new Error("An instance of Alfresco.module.CreateDatalist already exists.");
      }

      Alfresco.module.CreateDatalist.superclass.constructor.call(this, "Alfresco.module.CreateDatalist", instanceId, ["button", "container", "connection", "selector", "json"]);
      this.options.container=container;
      return this;
   };

   YAHOO.extend(Alfresco.module.CreateDatalist, Alfresco.component.Base,
   {

      /**
       * Shows the CreateDatalist dialog to the user.
       *
       * @method show
       */
      show: function CreateDatalist_show()
      {
         if (this.widgets.panel)
         {
            // Panel is already in the DOM, so just show it
            this._showPanel();
         }
         else
         {
            /**
             * Load the gui from the server and let the templateLoaded() method
             * handle the rest.
             */
            Alfresco.util.Ajax.request(
            {
               url: this.getTemplateUrl(),
               dataObj:
               {
                  htmlid: this.id
               },
               successCallback:
               {
                  fn: this.onTemplateLoaded,
                  scope: this
               },
               execScripts: true,
               failureMessage: "Could not load create site template"
            });
         }
      },
      getTemplateUrl: function CreateDatalist_getTemplateUrl() {
		   return Alfresco.constants.URL_SERVICECONTEXT + "lyse/modules/lyse-create-datalist";
	  },
	  getFormUrl: function CreateDatalist_getFormUrl() {
		   return Alfresco.constants.URL_SERVICECONTEXT + "lyse/modules/lyse-create-datalist";
	  },
      /**
       * Called when the CreateDatalist html template has been returned from the server.
       * Creates the YUI gui objects such as buttons and a panel and shows it.
       *
       * @method onTemplateLoaded
       * @param response {object} a Alfresco.util.Ajax.request response object
       */
      onTemplateLoaded: function CreateDatalist_onTemplateLoaded(response)
      {
         // Inject the template from the XHR request into a new DIV element
         var containerDiv = document.createElement("div");
         containerDiv.innerHTML = response.serverResponse.responseText;

         // The panel is created from the HTML returned in the XHR request, not the container
         var panelDiv = Dom.getFirstChild(containerDiv);

         this.widgets.panel = Alfresco.util.createYUIPanel(panelDiv);

         // Create the cancel button
         this.widgets.cancelButton = Alfresco.util.createYUIButton(this, "cancel-button", this.onCancelButtonClick);

         // Create the ok button, the forms runtime will handle when its clicked
         this.widgets.okButton = Alfresco.util.createYUIButton(this, "ok-button", null,
         {
            type: "submit"
         });

         // Configure the forms runtime
         var CreateDatalistForm = new Alfresco.forms.Form(this.id + "-form");
         this.widgets.form = CreateDatalistForm;

         var elTitle = Dom.get(this.id + "-title");

        this.widgets.typeObj = Dom.get(this.id + "-datalistType");

        var elDatalistContainer = Dom.get(this.id + "-datalistContainer");
        elDatalistContainer.value = this.options.container;

         /**
          * Title field
          */
         // Title is mandatory
         CreateDatalistForm.addValidation(elTitle, Alfresco.forms.validation.mandatory, null, "keyup", this.msg("validation-hint.mandatory"));
         // ...and has a maximum length
         CreateDatalistForm.addValidation(elTitle, Alfresco.forms.validation.length,
         {
            max: 256,
            crop: true
         }, "keyup");


         /**
          * Description field
          */

         // Description kept to a reasonable length
         CreateDatalistForm.addValidation(this.id + "-description", Alfresco.forms.validation.length,
         {
            max: 512,
            crop: true
         }, "keyup");




         // The ok button is the submit button, and it should be enabled when the form is ready
         CreateDatalistForm.setSubmitElements(this.widgets.okButton);
         CreateDatalistForm.doBeforeFormSubmit = {
            fn: this.doBeforeFormSubmit,
            obj: null,
            scope: this
         };

         // Submit as an ajax submit (not leave the page), in json format
         CreateDatalistForm.setAJAXSubmit(true,
         {
            successCallback:
            {
               fn: this.onCreateDatalistSuccess,
               scope: this
            },
            failureCallback:
            {
               fn: this.onCreateDatalistFailure,
               scope: this
            }
         });
         CreateDatalistForm.setSubmitAsJSON(true);
         // We're in a popup, so need the tabbing fix
         CreateDatalistForm.applyTabFix();
         CreateDatalistForm.doBeforeAjaxRequest = {
            fn: this.doBeforeAjaxRequest,
            scope: this
         };
         CreateDatalistForm.init();

         // Show the panel
         this._showPanel();
      },


      /**
       * Interceptor just before Ajax request is sent.
       *
       * @method doBeforeAjaxRequest
       * @param p_config {object} Object literal containing request config
       * @return {boolean} True to continue sending form, False to prevent it
       */
      doBeforeAjaxRequest: function CreateDatalist_doBeforeAjaxRequest(p_config)
      {
         return true;
      },

      /**
       * Called before the form is about to be submitted
       *
       * @method doBeforeFormSubmit
       * @param form {HTMLFormElement} The create site form
       * @param obj {Object} Callback object
       */
      doBeforeFormSubmit: function(form, obj)
      {
         var formEl = Dom.get(this.id + "-form");
         formEl.attributes.action.nodeValue = this.getFormUrl();

         this.widgets.cancelButton.set("disabled", true);


         this.widgets.panel.hide();
         this.widgets.feedbackMessage = Alfresco.util.PopupManager.displayMessage(
         {
            text: Alfresco.util.message("message.creating", this.name),
            spanClass: "wait",
            displayTime: 0
         });
      },



      /**
       * Called when user clicks on the cancel button.
       * Closes the CreateDatalist panel.
       *
       * @method onCancelButtonClick
       * @param type
       * @param args
       */
      onCancelButtonClick: function CreateDatalist_onCancelButtonClick(type, args)
      {
         // Reset the form fields
         try
         {
            Dom.get(this.id + "-title").value = "";
            Dom.get(this.id + "-description").value = "";
            Dom.get(this.id + "-datalistType").selectedIndex = 0;
         }
         catch(e)
         {
         }

         this.widgets.panel.hide();
      },

      /**
       * Called when a datalist has been succesfully created on the server.
       *
       * @method onCreateDatalistSuccess
       * @param response
       */
      onCreateDatalistSuccess: function CreateDatalist_onCreateDatalistSuccess(response)
      {
         if (response.json !== undefined && response.json.success)
         {
            Alfresco.util.PopupManager.displayMessage(
             {
                text: Alfresco.util.message("message.success")
             });

             // refresh datagrid of the datalist manger
             location.reload();
         }
         else
         {
            this._adjustGUIAfterFailure(response);
         }
      },

      /**
       * Called when a datalist failed to be created.
       *
       * @method onCreateDatalistFailure
       * @param response
       */
      onCreateDatalistFailure: function CreateDatalist_onCreateDatalistFailure(response)
      {
         this._adjustGUIAfterFailure(response);
      },

      /**
       * Helper method that restores the gui and displays an error message.
       *
       * @method _adjustGUIAfterFailure
       * @param response
       */
      _adjustGUIAfterFailure: function CreateDatalist__adjustGUIAfterFailure(response)
      {
         this.widgets.feedbackMessage.destroy();
         this.widgets.cancelButton.set("disabled", false);
         this.widgets.panel.show();
         var text = Alfresco.util.message("message.failure", this.name);

         if (response.serverResponse.status === 403)
         {
            // User does not have permissions to create the site
            if (response.json.message)
            {
               text = Alfresco.util.message(response.json.message, this.name)
            }
            else
            {
               text = Alfresco.util.message("error.noPermissions", this.name);
            }
         }
         else if (response.json.message)
         {
            var tmp = Alfresco.util.message(response.json.message, this.name);
            text = tmp ? tmp : text;
         }
         Alfresco.util.PopupManager.displayPrompt(
         {
            title: Alfresco.util.message("message.failure", this.name),
            text: text
         });
      },

      /**
       * Prepares the gui and shows the panel.
       *
       * @method _showPanel
       * @private
       */
      _showPanel: function CreateDatalist__showPanel()
      {
         // Reset form before displaying it
         this.widgets.form.reset();

         // Show the upload panel
         this.widgets.panel.show();

         // Firefox insertion caret fix
         Alfresco.util.caretFix(this.id + "-form");

         if (!this.widgets.escapeListener)
         {
            // Register the ESC key to close the dialog
            this.widgets.escapeListener = new KeyListener(document,
            {
               keys: KeyListener.KEY.ESCAPE
            },
            {
               fn: function(id, keyEvent)
               {
                  this.onCancelButtonClick();
               },
               scope: this,
               correctScope: true
            });
            this.widgets.escapeListener.enable();
         }

         // Set the focus on the first field
         Dom.get(this.id + "-title").focus();
      }
   });
})();

Alfresco.module.getCreateDatalistInstance = function(container)
{
   var instanceId = "alfresco-createDatalist-instance";
   return Alfresco.util.ComponentManager.get(instanceId) || new Alfresco.module.CreateDatalist(instanceId, container);
};
